function v = splinetx_natural(x,y,u)
% function v = splinetx_natural(x,y,u)
% This is a modification of the standard textbook spline function.
% It finds the piecewise cubic natural interpolatory spline
%     S(x), with S(x(j)) = y(j), and returns v(k) = S(u(k)).
%
% Inputs: x - data locations, which must be sorted in ascending order
%         y - data values at the data locations
%         u - locations at which to evaluate the spline
% Outputs: v - spline values at the locations u
%
%  See SPLINE, PCHIPTX.

%  First derivatives

   h = diff(x);
   delta = diff(y)./h;
   d = splineslopes_PRIVATE(h,delta);

%  Piecewise polynomial coefficients

   n = length(x);
   c = (3*delta - 2*d(1:n-1) - d(2:n))./h;
   b = (d(1:n-1) - 2*delta + d(2:n))./h.^2;

%  Find subinterval indices k so that x(k) <= u < x(k+1)

   k = ones(size(u));
   for j = 2:n-1
      k(x(j) <= u) = j;
   end

%  Evaluate spline

   s = u - x(k);
   v = y(k) + s.*(d(k) + s.*(c(k) + s.*b(k)));


% -------------------------------------------------------

function d = splineslopes_PRIVATE(h,delta)
%  SPLINESLOPES  Slopes for cubic spline interpolation.
%  splineslopes(h,delta) computes d(k) = S'(x(k)).
%  Uses natural end conditions.

%  Diagonals of tridiagonal system

   n = length(h)+1;
   a = zeros(size(h)); b = a; c = a; r = a;
   a(1:n-2) = h(2:n-1);
   a(n-1) = 1;
   b(1) = 2;
   b(2:n-1) = 2*(h(2:n-1)+h(1:n-2));
   b(n) = 2;
   c(1) = 1;
   c(2:n-1) = h(1:n-2);

%  Right-hand side

   r(1) = 3*delta(1);
   r(2:n-1) = 3*(h(2:n-1).*delta(1:n-2)+ ...
              h(1:n-2).*delta(2:n-1));
   r(n) = 3*delta(n-1);

%  Solve tridiagonal linear system
   A = gallery('tridiag',a,b,c);
   d = A\r;
